/** 
 * SALSA 2
 * July 2000
 * Mike Gasser (gasser@indiana.edu)
 */

import java.awt.*;
import java.awt.event.*;

/**
* Frame for displaying some information about all of the critters
* in the population.
*/
abstract class CritterFrame extends Frame
  implements WindowListener, MouseListener {

  public final static Color
    BG_COLOR = Color.white;
    
  protected UI ui;

  protected World world;

//  protected String helpText;
  
  protected int mouseIndex;
    
  // The canvas where the displaying really takes place.
  protected CritterCanvas canvas;
  
  CritterFrame (UI ui, World world, String title) {
    this.ui = ui;
    this.world = world;
    canvas = makeCanvas();
    canvas.addMouseListener (this);
    setLayout (new GridLayout (1, 1));
    add (canvas);
    
    addWindowListener (this);
    addMouseListener (this);

    setBackground(BG_COLOR);
    setTitle (title);
//    setMenuBar(ui.getMenuBar());
    pack();
    setVisible (true);
    // Make this a window that can be selected in the Windows menu.
    ui.addWindow (this);
  } // constructor CritterFrame
  
  public CritterCanvas getCanvas() {
      return canvas;
  } // method getCanvas

  public void repaint() {
      canvas.repaint();
  }

  public void windowClosed (WindowEvent event) {
  } // method windowClosed

  public void windowDeiconified (WindowEvent event) {
  } // method windowDeiconified

  public void windowIconified (WindowEvent event) {
  } // method windowIconified

  public void windowActivated (WindowEvent event) {
  } // method windowActivated

  public void windowDeactivated (WindowEvent event) {
  } // method windowDeactivated

  public void windowOpened (WindowEvent event) {
  } // method windowOpened

  public void windowClosing (WindowEvent event) {
    ui.removeWindow (this);
  } // method windowClosing

  public void mouseClicked (MouseEvent event) {
  } // method mouseClicked

  /**
  * When the mouse is pressed, highlight the critter in the
  * world frame.
  */
  public void mousePressed (MouseEvent event) {
      mouseIndex = event.getY() / canvas.getCellHeight();
      if (mouseIndex < world.getNCritters())
        world.showCritter(mouseIndex, true);
  } // method mousePressed
    
  public void mouseReleased (MouseEvent event) {
      if (mouseIndex < world.getNCritters())
        world.showCritter(mouseIndex, false);
  } // method mouseReleased

  /**
  * The window should provide help.
  */
  public void mouseEntered (MouseEvent event) {
      /*
    if (ui.showHelp) {
      ui.helpWrite(helpText);
    }
    */
  } // method mouseEntered
  
  public void mouseExited (MouseEvent event) {
      /*
    if (ui.showHelp) {
      ui.helpWrite(UI.HELP_INSTRUCTIONS);
    }
    */
  } // method mouseExited
  
  abstract protected CritterCanvas makeCanvas();

} // class CritterFrame
