/** 
 * SALSA 1.1
 * June 2000
 * Mike Gasser (gasser@indiana.edu)
 */

import java.io.*;
import java.awt.Color;
import java.awt.Graphics;

public class Touch extends Sensor {

  private final static Color
    BORDER_COLOR = Color.white,
    EMPTY_COLOR = Color.black,
    HARD_COLOR = Cell.MEDIUM_BLUE,
    SOFT_COLOR = Cell.LIGHT_GREEN,
    ROUGH_COLOR = Color.green,
    SQUISHY_COLOR = Color.red;

  private int
    range;
    
  private int[]
    feels;
    
  public int
    // Index of the Critter; needed to get the right row in
    // touchCanvas.
    index = 0;
    
  Touch (UI ui, int species) {
    super (ui, species);
//    nStates = Params.getNFeels();
    nStates = Params.FEELS.length;
    range = getSpecies().get(Params.TOUCH_RANGE);
    length = nStates * range;
    feels = new int[range];
  } // constructor Touch
  
  public int getUnitsPerPosition() {
      return nStates;
  } // getUnitsPerPosition()
  
  protected Color getColor(int slot) {
      return feelToColor(Params.getFeel(slot));
  }
  
  protected Color getDisplayColor(int i) {
      return feelToColor(getFeel(i));
  } // method getDisplayColor(int)
  
  protected void setDisplayLength() {
      displayLength = getSpecies().get(Params.TOUCH_RANGE);
  } // method setDisplayLength
  
  private void setFeel(int position, int feel) {
      feels[position] = feel;
  } // method setFeel(int, int)
  
  private void setFeels(Cell cell, int direction) {
      int start = Heading.add(direction, 6 - range / 2);
      int i, d;
      for (i = 0, d = start; i < range; i++, d = Heading.add(d, 1))
	  setFeel(i, cell.getNextFeel(d));
  } // method setFeels(Cell, int)
  
  public int getFeel(int position) {
      return feels[position];
  } // method getFeel(int)
  
  public int[] sense(Cell cell, int direction) {
      int feel;
      int c, f, i;
      int[] sensed = new int[length];
      
      setFeels(cell, direction);
      
      for (c = 0, i = 0; c < range; c++) {
          feel = getFeel(c);
	  for (f = 0; f < nStates; f++, i++) {
	      if (feel == Params.getFeel(f))
	        sensed[i] = 1;
	      else
		sensed[i] = 0;
	  }
      }
      return sensed;
  } // method sense(Cell, direction)

  protected void paint(Graphics g, int i, int x, int y, int w, int h) {
      g.fillRect(x + 1, y + 1, w - 1, h - 1);
  } // method point(Graphics, int, int, int, int, int)

  protected void clear(Graphics g, int x, int y, int w, int h) {}

  private static Color feelToColor(int feel) {
      if (feel == Cell.EMPTY)
	  return EMPTY_COLOR;
      else if (feel == Cell.HARD)
	  return HARD_COLOR;
      else if (feel == Cell.SOFT)
	  return SOFT_COLOR;
      else if (feel == Cell.SQUISHY)
	  return SQUISHY_COLOR;
      else if (feel == Cell.ROUGH)
	  return ROUGH_COLOR;
      else
	  return EMPTY_COLOR;
  } // method feelToColor(int)

} // class Sensor
