/** 
 * SALSA 1.1
 * June 2000
 * Mike Gasser (gasser@indiana.edu)
 **/

import java.awt.*;
import java.awt.event.*;

/**
* The Canvas within the WorldFrame where the World grid and
* the Things in the World actually get drawn.  Gets created
* whenever WorldFrame is instantiated.
*/

public class WorldCanvas extends Canvas {

  private final static Color
    BG_COLOR = Color.black,
    COLLISION_COLOR = Color.yellow;

  private int
    width, height,
    rows, columns,
    cellSize,
    cellIncr;
  
  private World world;
  private UI ui;
  
  private Image image;
  private Graphics ig;

  WorldCanvas (World world, UI ui, int columns, int rows,
    int cellSize, int cellIncr) {
      this.world = world;
      world.canvas = this;
      this.ui = ui;
      this.rows = rows;
      this.columns = columns;
      this.cellSize = cellSize;
      this.cellIncr = cellIncr;
      this.width = (3 * columns + 1) * cellSize;
      this.height = (2 * rows + 1) * cellIncr +1;
      // Make this the canvas for all cells
      setBackground (BG_COLOR);
      world.addCanvas(this);
//      ui.write("World width : " + width + ", height: " + height);
  } // constructor WorldCanvas
  
  public Dimension getMinimumSize() {
    return new Dimension (width, height);
  } // method getMinimumSize

  public Dimension getPreferredSize() {
    return getMinimumSize();
  } // method getPreferredSize
  
  public int getWidth() {
      return width;
  } // method getWidth
  
  public int getHeight() {
      return height;
  } // method getHeight
  
  private void initImage() {
    image = createImage(width, height);
    ig = image.getGraphics();
  } // method initImage()
  
  /**
  * Find the Cell with the given pixel coordinates, null if
  * this is out of bounds.   
  */
  public Cell getSelectedCell(int[] coords) {
    int x = coords[0], y = coords[1];
    
    int gridX,
        gridY;
    Cell cell0,
         cell = null;
    boolean selected = false;
    
    if (x < cellSize)
	gridX = Params.getWorld(Params.NCOLUMNS) - 1;
    else
	gridX = (int)(x / (3 * cellSize));
	
    if (gridX % 2 == 0)
      gridY = (int)(y / (2 * cellIncr));
    else {
      gridY = (int)((y + cellIncr) / (2 * cellIncr));
      gridY--;
    }
  
//    ui.write ("GridX: " + gridX + ", gridY: " + gridY);

    cell0 = world.getCell(gridX, gridY);
    
    // Try the neighbors of cell0 and see if they're closer.
    int c = 0;
    while (!selected && c < 6) {
	cell = cell0.getNeighbor (c);
	if (cell != null && cell.contains (x, y))
	    selected = true;
	c++;
    }
    
    if (selected == true)
      return (cell);
    else
      return (cell0);
  } // method getSelectedCell
  
  public void clear() {
      clear(getGraphics());
  } // clear()
  
  public void clear(Graphics g) {
      g.clearRect (0, 0, width, height);
  } // method clear
  
  public void paint (Graphics g) {
    if (image == null)
	initImage();
    
    world.paint(ig);

    g.drawImage(image, 0, 0, this);
  } // method paint(Graphics)

} // class WorldCanvas
