/** 
 * SALSA 1.1
 * June 2000
 * Mike Gasser (gasser@indiana.edu)
 */

import java.io.*;
import java.util.Vector;

public class Species {
    
  private static final double
    NINPUTS_PER_DECR = 25.0;
    
  public static final int
    NPARAMS = 20,
    STRENGTH_SCORE_DECR = 200,
    VALUE = 0,
    MINIMUM = 1,
    MAXIMUM = 2;
    
  public static int[][]
    VALUES = new int[NPARAMS][3];
    
  private UI ui;
  
  private int[] values;
  
  private int
    nInputs,
    nOutputs,
    nConnections,
    stepDecr;
    
  private double
    addStepDecr;
   
  // Only used when a Critter is saved as a Species
  private int age = 0;
  private Memory memory = null;
  
  private Vector members = new Vector();
  
  Species(UI ui, int[] values) {
    this.ui = ui;
    this.values = values;
  } // Species(UI, values)

  // Getter methods.
  
  public static int getDefault(int i) {
      return VALUES[i][VALUE];
  }
  
  public static int getMax(int i) {
      return VALUES[i][MAXIMUM];
  }
  
  public static int getMin(int i) {
      return VALUES[i][MINIMUM];
  }
  
  public int get(int i) {
      return values[i];
  }
  
  public int getAge() {
      return age;
  } // getAge()
  
  /**
  * Converts the int to a char.
  */
  public char getName() {
      return (char)(get(Params.NAME));
  } // getName()
  
  public Memory getMemory() {
      return memory;
  } // getMemory()
  
  private Critter getMember(int i) {
      return (Critter)(members.elementAt(i));
  }
  
  private int getNMembers() {
      return members.size();
  }
  
  public boolean hasSensor(int type) {
      if (type == Params.TOUCH)
	  return (get(Params.TOUCH_RANGE) > 0);
      else if (type == Params.VISION)
	  return (get(Params.VISUAL_ANGLE) > 0);
      else
	  return (get(Params.SONAR_ANGLE) > 0);
  } // hasSensor(int)
  
  // Setter methods
  
  public void setMemory(Memory m) {
      memory = m;
  } // setMemory(Memory)
  
  public void setAge(int a) {
      age = a;
  } // setAge(int)
  
  private void setNInputs() {
      nInputs = 0;
      if (hasSensor(Params.TOUCH))
	  nInputs += (get(Params.TOUCH_RANGE) * Params.FEELS.length);
      if (hasSensor(Params.VISION))
	  nInputs += (Params.COLORS.length *
//		      get(Params.NCOLORS) *
		  get(Params.VISUAL_RESOLUTION));
      if (hasSensor(Params.SONAR))
	  nInputs += (get(Params.SONAR_RESOLUTION) *
		  get(Params.NSONAR));
//      ui.write("NInputs: " + sum);
  } // setNInputs()
  
  private void setNOutputs() {
      nOutputs = get(Params.NREST) +
	         get(Params.NTURN) +
	         get(Params.NMOVE) +
	         get(Params.NEAT) +
	         get(Params.NHIT);
  } // setNOutputs()
  
  // Setter methods.
  
  public void set(int i, int val) {
      values[i] = val;
  }
  
  public static void set(int i, int type, int value) {
      VALUES[i][type] = value;
  }
  
  public static void set(int index, int def, int min, int max) {
      set(index, VALUE, def);
      set(index, MAXIMUM, max);
      set(index, MINIMUM, min);
  }

  public static void set() {
      // Sensors
      set(Params.TOUCH_RANGE, -1, -1, 6);
      
      set(Params.VISUAL_ANGLE, -1, -1, 360);
      set(Params.VISUAL_RESOLUTION, 3, 1, 6);
      // Not currently set in params file
      set(Params.NCOLORS, 4, 4, 4);
      
      set(Params.SONAR_RESOLUTION, 3, 1, 6);
      set(Params.SONAR_ANGLE, -1, -1, 360);
      // Not currently set in params file
      set(Params.NSONAR_DISTINCTIONS, 4, 4, 4);
      
      // Effectors
      set(Params.NREST, 0, 0, 1);
      set(Params.NMOVE, 1, 1, 3);
      set(Params.NTURN, 2, 1, 5);
      set(Params.NEAT, 1, 1, 1);
      set(Params.NHIT, 1, 0, 1);
      
      // Learning and action parameters
      set(Params.LEARNING_RATE, 5, 0, 100);
      set(Params.DISCOUNT_RATE, 5, 0, 10);
      set(Params.EXPLOITATION_RATE, 10, 0, 1000);
      
      // Species
      set(Params.NAME, 0, 0, 99);
      set(Params.N, 5, 0, 25);
  } // set()
  
  private void setMemory() {
      setNInputs();
      setNOutputs();
      nConnections = nInputs + nOutputs;
  } // setMemory()
  
  private void setStepDecr() {
      double decr = Params.getWorld(Params.STEP_DECR) * nInputs /
                    NINPUTS_PER_DECR;
      // Round towards 0.
      stepDecr = (int)decr;
      // Absolute value of what's left over.
      addStepDecr = stepDecr - decr;
  } // setNDecr();
  
  public void addMember(Critter c) {
      members.addElement(c);
  }
  
  public void init() {
    setMemory();
    // If all plants need processing, enable hit.
    if (Params.getWorld(Params.PLANT_PLACEMENT) == Params.TOUGH)
	set(Params.NHIT, 1);
//    setStepDecr();
  } // init()
  
  public void describe() {
     ui.write("Critter species: " + (char)(get(Params.NAME)));
     ui.write("  Number of critters: " + get(Params.N));
     ui.write("  Number of input (sensor) units: " + nInputs);
     ui.write("  Number of output (action) units: " + nOutputs);
//     ui.write("  Number of connections: " + nConnections);
//     ui.write("  Step decrement: " + stepDecr);
//     ui.write("  Add step decrement: " + addStepDecr);
  } // describe()
  
  public void describeMembers(PrintWriter out, int tSteps) {
      int n = getNMembers();
      int m;
      int age;

      int maxAge = 0;

      int ages = 0;
      int strengths= 0;
      int reincarnations = 0;
      
      double reincMean = 0.0;
      double score, score2;
      
      for (m = 0; m < n; m++) {
	  age = getMember(m).getRealAge();

	  reincarnations += getMember(m).getNReincarnations();
	  strengths += getMember(m).getStrengthValue();
	  ages += age;
	  if (age > maxAge)
	      maxAge = age;
      }
      ages /= n;
      strengths /= n;
      reincMean = ((double)reincarnations) / n;
      score = getScore(tSteps, reincMean, strengths);
//      score2 = getScore2(tSteps, reincMean, strengths, maxAge);
      
      ui.write("Critter species: " + getName());
      ui.write("  Mean age: " + ages);
      ui.write("  Max age: " + maxAge);
      ui.write("  Mean strength: " + strengths);
      ui.write("  Mean reincarnations: " + reincMean);
      ui.write("  Score: " + score);
//      ui.write("  Score2: " + score2);
      if (out != null) {
	  out.println("  Critter species: " + getName());
	  out.println("    Mean age: " + ages);
	  out.println("    Max age: " + maxAge);
	  out.println("    Mean strength: " + strengths);
	  out.println("    Mean reincarnations: " + reincMean);
	  out.println("    Score: " + score);
//	  out.println("    Score2: " + score2);
      }
  } // describeMembers(PrintWriter, int)

  public int getStepDecr() {
      if (Math.random() > addStepDecr) {
	  return stepDecr;
      } else {
	  return stepDecr - 1;
      }
  } // getStepDecr()
  
  public static double getScore(int tSteps, double nReincs, int str) {
      return tSteps /
       ((Strength.MAX * nReincs) + Strength.MAX + STRENGTH_SCORE_DECR - str);
  }  // getScore(int, double, int)
  
  public static double getScore2(int tSteps, double nReincs, int str, int maxAge) {
      return (tSteps + maxAge) /
       ((Strength.MAX * nReincs) + Strength.MAX + STRENGTH_SCORE_DECR - str);
  } // getScore2(int, double, int, int)

} // class Species
